

/************************************************************************/
/*                                                                      */
/*    WebServer                                                         */
/*      Version 4                                                                */
/*    A Example chipKIT HTTP Server implementation                      */
/*    This sketch is designed to work with web browser clients          */
/*                                                                      */
/************************************************************************/
/*    Author:     Keith Vogel                                           */
/*    Copyright 2014, Digilent Inc.                                     */
/************************************************************************/
/* 
*
* Copyright (c) 2013-2014, Digilent <www.digilentinc.com>
* Contact Digilent for the latest version.
*
* This program is free software; distributed under the terms of 
* BSD 3-clause license ("Revised BSD License", "New BSD License", or "Modified BSD License")
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* 1.    Redistributions of source code must retain the above copyright notice, this
*        list of conditions and the following disclaimer.
* 2.    Redistributions in binary form must reproduce the above copyright notice,
*        this list of conditions and the following disclaimer in the documentation
*        and/or other materials provided with the distribution.
* 3.    Neither the name(s) of the above-listed copyright holder(s) nor the names
*        of its contributors may be used to endorse or promote products derived
*        from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/************************************************************************/
/*  Revision History:                                                   */
/*                                                                      */
/*    6/15/2014(KeithV): Created                                        */
      //edits 3/25/16 Bryce Johnson
/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  This HTTP Server Example contains sample content in the content     */
/*  subdirectory under the sketch directory. The files in the content   */
/*  directory should be copied to the root of your SD card. However,    */
/*  you can view the content by opening HomePage.htm page               */
/*  with your browser right out of the content subdirectory before      */
/*  loading it on the SD card. The sample pages contain                 */
/*  instructions on how to set up this example and therefore may be     */
/*  valuable to read before proceeding.                                 */
/*                                                                      */
/************************************************************************/

//************************************************************************
//************************************************************************
//**************************  READ THIS  *********************************
//************************************************************************
//***********  !! COPY THIS SKETCH TO YOUR SKETCH DIRECTORY !! ***********
//************************************************************************
//************************************************************************
//************** You will not be able to modify the network **************
//************** parameters as long as this sketch is in a  **************
//************* library subdirectory. Even if you modify in **************
//************* MPIDE it will not take effect when you build  ************
//************** and you will not be able to connect to ******************
//************** your router, the connection will timeout ****************
//************************************************************************
//************************************************************************

/************************************************************************/
/*                       Supported hardware:                            */
/*                                                                      */
/*  uC32 with a WiFiShield                                              */
/*  WF32                                                                */
/*  WiFIRE                                                              */
/*  MX7cK with a pmodSD on JPF                                          */
/*                                                                      */
/*  NOTE: you can NOT stack a NetworkShield and a WiFiShield on a Max32 */
/************************************************************************/

//************************************************************************
//************************************************************************
//********  SET THESE LIBRARIES FOR YOUR HARDWARE CONFIGURATION **********
//************************************************************************
//************************************************************************

/************************************************************************/
/*                                                                      */
/*    Network Hardware libraries                                        */
/*    INCLUDE ONLY ONE                                                  */
/*                                                                      */
/************************************************************************/
// You MUST select 1 and ONLY 1 of the following hardware libraries
// they are here so that MPIDE will put the lib path on the compiler include path.
#include <MRF24G.h>                         // This is for the MRF24WGxx on a pmodWiFi or WiFiShield
//#include <IM8720PHY.h>                      // This is for the the Internal MAC and SMSC 8720 PHY

/************************************************************************/
/*    Network libraries                                                 */
/************************************************************************/
// The base network library
// this is a required library
// Do not comment out this library
#include <DEIPcK.h>

//  -----  COMMENT THIS OUT IF YOU ARE NOT USING WIFI  -----
#include <DEWFcK.h>

//************************************************************************
//************************************************************************
//**************** END OF LIBRARY CONFIGURATION **************************
//************************************************************************
//************************************************************************

/************************************************************************/
/*                                                                      */
/*           YOU MUST.....                                              */
/*                                                                      */
/*    You MUST put HTTPServerConfig.h in your sketch director           */
/*    And you MUST configure it with your network parameters            */
/*                                                                      */
/*    You also MUST load your content onto your SD card and             */
/*    the file HomePage.htm MUST exist at the root of the SD            */
/*    file structure. Of course you must insert your SD card            */
/*    into the SD reader on the chipKIT board                           */
/*                                                                      */
/*    Go do this now....                                                */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*    Other libraries; Required libraries                               */
/************************************************************************/
// You must have an SD card reader somewhere
// as the HTTP server uses the SD card to hold the HTML pages.
// this is a required library; we must specify this in the .pde for
// MPIDE to include it.
#include    <SD.h>


//need servo library
#include <Servo.h>

// and this is the HTTPServer library code.
// this is a required library
#include    <HTTPServer.h>

/************************************************************************/
/*    HTTP URL Matching Strings                                         */
/************************************************************************/
// These are the HTTP URL match strings for the dynamically created
// HTML rendering functions.
// Make these static const so they get put in flash
static const char szHTMLRestart[]       = "GET /Restart ";
static const char szHTMLTerminate[]     = "GET /Terminate ";
static const char szHTMLReboot[]        = "GET /Reboot ";
static const char szHTMLFavicon[]       = "GET /favicon.ico ";
static const char szHTMLGetPins[]       = "GET /PinsPage.htm ";
static const char szHTMLPostPins[]      = "POST /PinsPage.htm ";

// here is our sample/example dynamically created HTML page
GCMD::ACTION ComposeHTMLGetPINS(CLIENTINFO * pClientInfo);
GCMD::ACTION ComposeHTMLPostPINS(CLIENTINFO * pClientInfo);

// get rid of as much of the heap as we can, the SD library requires some heap
#define CHANGE_HEAP_SIZE(size) __asm__ volatile ("\t.globl _min_heap_size\n\t.equ _min_heap_size, " #size "\n")
CHANGE_HEAP_SIZE(0x200);

//create servo instances here/place motor enable and direction on correct pin
Servo pan; //left and right
Servo tilt; //pan up and down


#define pan_servo_pin 31
#define tilt_servo_pin 30


//global servo positions

//create a global count variable
//this will allow us to only update the servo and motor positions every so many times through the process server loop
int count=0;

//GLOBAL #DEFINES
#define DIR_UP 1
#define DIR_DOWN 0
#define START 250
#define STOP 0
#define STEP_UP 5
#define STEP_DOWN 1


//define maximum tilts and pans (as mount dependent)
#define max_tilt_down 147
#define max_tilt_up 62
#define max_pan_right 1
#define max_pan_left 134


//pins and limit switches
#define top_limit_pin 36
#define bottom_limit_pin 38
#define on_off_switch_pin 37

// define all of these, enable easy changes in the future.......................................
#define tilt_up_pin 3
#define tilt_down_pin 2
#define pan_right_pin 5
#define pan_left_pin 4
#define height_up_pin 6
#define height_down_pin 7
#define home_pin 8
#define scan_pin 9
#define motor_enable_pin 10
#define motor_direction_pin 11

#define step_pin_1 32
#define step_pin_2 33
#define step_pin_3 34
#define step_pin_4 35

int tilt_up_previous_state;
int tilt_down_previous_state;
int pan_right_previous_state;
int pan_left_previous_state;
int height_up_previous_state;
int height_down_previous_state;
int home_previous_state;
int tilt_position=90;
int pan_position=90;
int scan_previous_state; //low is not scanning, high is scanning
int scan_direction;
int initialization_state;
int finished=false;
int still_going=true;

/***    void setup(void)
 *
 *    Parameters:
 *          None
 *              
 *    Return Values:
 *          None
 *
 *    Description: 
 *    
 *      Arduino Master Initialization routine
 *      
 *      
 * ------------------------------------------------------------ */
void setup(void) 
{   
    // Must do a Serial.begin because the HTTP Server
    // has diagnostic prints in it.
    Serial.begin(9600);
    Serial.println("Mount Control V5");
    Serial.println("Written by Bryce Johnson with use of adapted code written by Keith Vogel");

    Serial.println();

    InitializePins();

    // add rendering functions for dynamically created web pages
    // max of 10 AddHTMLPage() allowed 

    // these are the Select picture pages in Post.htm
    AddHTMLPage(szHTMLGetPins,      ComposeHTMLGetPINS);
    AddHTMLPage(szHTMLPostPins,     ComposeHTMLPostPINS);

    // comment this out if you do not want to support
    // restarting the network stack from a browser
    AddHTMLPage(szHTMLRestart,      ComposeHTMLRestartPage);

    // comment this out if you do not want to support
    // terminating the server from a browser
    AddHTMLPage(szHTMLTerminate,    ComposeHTMLTerminatePage);

    // comment this out if you do not want to support
    // rebooting (effectively hitting MCLR) the server from a browser
    AddHTMLPage(szHTMLReboot,       ComposeHTMLRebootPage);

    // This example supports favorite ICONs, 
    // those are those icon's next to the URL in the address line 
    // on the browser once the page is displayed.
    // To support those icons, have at the root of the SD file direcotory
    // an ICON (.ico) file with your ICON in it. The file MUST be named
    // favicon.ico. If you do not have an icon, then uncomment the following
    // line so the server will tell the browser with an HTTP file not found
    // error that we don't have a favoite ICON.
    // AddHTMLPage(szHTMLFavicon,      ComposeHTTP404Error);

    // Make reading files from the SD card the default compose function
    SetDefaultHTMLPage(ComposeHTMLSDPage);

    // Initialize the SD card
    SDSetup();

    // Initialize the HTTP server
    ServerSetup();
    
    
    //setup limit switch pins and on/off switch
    pinMode(top_limit_pin,INPUT);
    pinMode(bottom_limit_pin, INPUT);
    pinMode(on_off_switch_pin, INPUT);  
    
    //set motor direction pin to digital out, motor enable pin to analog out
    pinMode(motor_direction_pin,OUTPUT);
    pinMode(motor_enable_pin, OUTPUT);
    
    //stepper motor buttons
    pinMode(step_pin_1,OUTPUT);
    pinMode(step_pin_2,OUTPUT);
    pinMode(step_pin_3,OUTPUT);
    pinMode(step_pin_4,OUTPUT);
    
    
    //place servos on the correct pins
    tilt.attach(tilt_servo_pin);
    tilt.write(90); //make sure centered at startup
    tilt_position=90;
    pan.attach(pan_servo_pin);
    pan.write(86);
    pan_position=86;
    
    //need to ensure that mount is at bottom here
    
    sendMountHome();

    
    //initialize states of pins
    tilt_up_previous_state=LOW;
    tilt_down_previous_state=LOW;
    pan_right_previous_state=LOW;
    pan_left_previous_state=LOW;
    height_up_previous_state=LOW;
    height_down_previous_state=LOW;
    home_previous_state=LOW;
    scan_previous_state=LOW;
    scan_direction=HIGH;
    initialization_state=still_going;
    still_going=true;
    finished=false;
    
    //make sure pins are in assumed state
    
}

/***    void loop(void) 
 *
 *    Parameters:
 *          None
 *              
 *    Return Values:
 *          None
 *
 *    Description: 
 *    
 *      Arduino Master Loop routine
 *      
 *      
 * ------------------------------------------------------------ */
void loop(void) 
{
    // process the HTTP Server
    ProcessServer();

     
    //move servos based on pin states, update only every 600 times through loop though
    if(count==2000 && digitalRead(on_off_switch_pin)==LOW && finished==true) //checks to make sure physical switch is on
    {

     updateServoPositions();
 
     count=0;
  
     
    }
    else if(count==2000 && digitalRead(on_off_switch_pin)==HIGH && finished==true)
    {
      count=0; 
      Serial.println("in switch off");
      //set current states of pins as values in global states
      //this will prevent movement when the master switch is turned back on
      tilt_up_previous_state=digitalRead(tilt_up_pin);
      tilt_down_previous_state=digitalRead(tilt_down_pin);
      pan_right_previous_state=digitalRead(pan_right_pin);
      pan_left_previous_state=digitalRead(pan_left_pin);
      height_up_previous_state=digitalRead(height_up_pin);
      height_down_previous_state=digitalRead(height_down_pin);
      home_previous_state=digitalRead(home_pin);
      scan_previous_state=digitalRead(scan_pin);
      
      
      
      count=0;
    }
    else if(still_going==true && count==2000)
    {
      
      //check to see if finished initializing
      //if all pins are set to digital low, then should be done, otherwise not
      if(digitalRead(tilt_up_pin)==LOW && digitalRead(tilt_down_pin)==LOW && digitalRead(pan_right_pin)==LOW && digitalRead(scan_pin)==LOW)
      {
        still_going=false;
        finished=true;
      }
      count=0;
    }
    count++;
}

int continueScan(void)
{
  Serial.print("scan prev state: ");
  Serial.println(scan_previous_state);
  if(scan_previous_state==LOW) //need to initialize if true
  {
          //make sure scan direction is going from left to right (high)
          scan_direction=HIGH;
          
          //make sure mount pointing far left
          pan.write(max_pan_left);
          pan_position=max_pan_left;
          
          //make sure mount tilted all the way down
          tilt.write(max_tilt_down);
          tilt_position=max_tilt_down;
          
          //send mount to full height (or make sure it is already there)
         // analogWrite(motor_enable_pin,STOP);
         // digitalWrite(motor_direction_pin, DIR_UP);
         // analogWrite(motor_enable_pin, START);
          int counter1=0;
          while((digitalRead(top_limit_pin)==HIGH)&&counter1<1200)
          {
            //motor continues to run
            counter1++;
            //delay(100);
            stepper_motor(STEP_UP);
            
          }
         // analogWrite(motor_enable_pin, STOP); //motor must be at top, thus stop
         stepper_motor(STOP);
         scan_previous_state=HIGH;
  }
  else //must still be in scan mode from before
  {
    //left to right
    if(scan_direction==HIGH)
    {
     // Serial.println("high,dir");
      //increment pan position to the right
      pan_position=pan_position - 8;
   //   Serial.print("Pan Position: ");
    //  Serial.println(pan_position);
    
      //make sure pan is not full right, if so tilt up more, check tilt
      if(pan_position<=max_pan_right)
      {
          pan_position=max_pan_right; //set to still within range
          
          tilt_position=tilt_position - 25; //increase tilt position
          //Serial.print("Tilt Position: ");
       //   Serial.println(tilt_position);
          if(tilt_position<=max_tilt_up)
          {
            //already went through scan, start over, by simply changing scan previous state to LOW
            scan_previous_state=LOW;
            Serial.println("should be exiting");
            return 1; //get out of function
          }
          //change pan direction to right to left (low)
          scan_direction=LOW;
      }
    }
      //right to left
    else if(scan_direction==LOW)
    {
      Serial.println("low,dir");
      //increment pan position to the left
      pan_position=pan_position + 8;
    //  Serial.print("Pan Position: ");
   //   Serial.println(pan_position);
    
      //make sure pan is not full left, if so tilt up more, check tilt
      if(pan_position>=max_pan_left)
      {
          pan_position=max_pan_left; //set to still within range
          
          tilt_position=tilt_position - 25; //increase tilt position
         // Serial.print("Tilt Position: ");
        //  Serial.println(tilt_position);
          if(tilt_position<=max_tilt_up)
          {
            //already went through scan, start over, by simply changing scan previous state to LOW
            scan_previous_state=LOW;
            return 1; //get out of function
          }
          //change pan direction to right to left (low)
          scan_direction=HIGH;
      }
    }
  
       //set servos to position
      pan.write(pan_position);
      tilt.write(tilt_position);
  }
  
  return 1;
}


void sendMountHome(void)
{
  //center servos
  pan.write(86);
  tilt.write(90);
  pan_position=86;
  tilt_position=90;
  
  //ensure motor stopped
  //analogWrite(motor_enable_pin, STOP);
  
  //set direction to down
 // digitalWrite(motor_direction_pin, DIR_DOWN);
  
  //start motor
 // analogWrite(motor_enable_pin, START);
  
  
  int bottom_limit_swith=digitalRead(bottom_limit_pin);
  
  int home_count=0;
  while(bottom_limit_swith==HIGH && home_count<1200)
  {
    //continue to run motor
    bottom_limit_swith=digitalRead(bottom_limit_pin);
    stepper_motor(STEP_DOWN);
    home_count++;
  }
  
  //stop motor
 // analogWrite(motor_enable_pin, STOP);
  stepper_motor(STOP);
}

//this function looks at the state of pins and from the defined values, determines which position to put the servos in. 
void updateServoPositions(void)
{  
  
  int tilt_up=LOW;
  int tilt_down=LOW;
  int pan_right=LOW;
  int pan_left=LOW;
  int height_up=LOW;
  int height_down=LOW;
  int home_state=LOW;
  int scan_state=LOW;
  
  tilt_up=digitalRead(tilt_up_pin);
  tilt_down=digitalRead(tilt_down_pin);
  pan_right=digitalRead(pan_right_pin); 
  pan_left=digitalRead(pan_left_pin); 
  height_up=digitalRead(height_up_pin); 
  height_down=digitalRead(height_down_pin);
  home_state=digitalRead(home_pin);
  scan_state=digitalRead(scan_pin);
  
  if(scan_state==HIGH)
  {
   
    continueScan();
  }
 
 //####################### tilt control #######################################
  else if(tilt_down!=tilt_down_previous_state) //down arrow key hit, want to tilt down
  {
    // must be a change, thus increment
    tilt_position=tilt_position + 20; //moves 20 degrees down
    if(tilt_position>max_tilt_down)
    {
      tilt_position=max_tilt_down; //ensures design not over max
    }
    tilt.write(tilt_position); 
    Serial.print("Tilt Position: ");
    Serial.println(tilt_position);
    tilt_down_previous_state=tilt_down; //update previous state
  }
  
  else if(tilt_up!=tilt_up_previous_state) //up arrow key hit, want to tilt up
  {
    // must be a change, thus increment
    tilt_position=tilt_position - 20; //moves 20 degrees upward
    if(tilt_position<max_tilt_up)
    {
      tilt_position=max_tilt_up; //ensures design not over max
    }
    tilt.write(tilt_position); 
    Serial.print("Tilt Position: ");
    Serial.println(tilt_position);
    tilt_up_previous_state=tilt_up;
  }
  //####################### END tilt control #######################################
  

  
  //######################################### pan control ##################################################
  
  else if(pan_left!=pan_left_previous_state) //left arrow key hit, want to pan left
  {
    // must be a change, thus increment
    pan_position=pan_position + 20; //moves 20 degrees left
    if(pan_position>max_pan_left)
    {
      pan_position=max_pan_left; //ensures design not over max
    }
    pan.write(pan_position); 
    
    pan_left_previous_state=pan_left;
  }
  
  else if(pan_right!=pan_right_previous_state) //left arrow key hit, want to pan left
  {
    // must be a change, thus increment
    pan_position=pan_position - 20; //moves 20 degrees upward
    if(pan_position<max_pan_right)
    {
      pan_position=max_pan_right; //ensures design not over max
    }
    pan.write(pan_position); 
    pan_right_previous_state=pan_right; //update previous state
  }
  // ############################################# end PAN control #################################
  
  
  //finish this section #######################################################************************************
  //####################################### start height control ###################################
else if(height_up!=height_up_previous_state) //8 key hit, want to go 
  {
    Serial.println("In HUP");
    // must be a change, thus move up
    //stop motor
    //analogWrite(motor_enable_pin,STOP);
    //set direction to up
    //digitalWrite(motor_direction_pin,DIR_UP);
    //start motor
    //analogWrite(motor_enable_pin, START); //values goe from 0 to 255, with 255 being a 100% duty cycle
    

     int up_count=0;
     //moves up for some to be determined time
     while((digitalRead(top_limit_pin)==HIGH) && (up_count<200))
     {
      // Serial.print("Top Limit State: ");
     //  Serial.println((digitalRead(top_limit_pin)));
       //delay(100);
       up_count++;
       stepper_motor(STEP_UP);
      
     }
     
  //stop motor
  //analogWrite(motor_enable_pin, STOP);
   stepper_motor(STOP);
     
    height_up_previous_state=height_up; //update previous state
  }
  
  else if(height_down!=height_down_previous_state) //2 arrow key hit, want to go down
  {
    //ensure motor stopped
    //analogWrite(motor_enable_pin,STOP);
    //set direction to up
    //digitalWrite(motor_direction_pin,DIR_DOWN);
    //start motor
   // analogWrite(motor_enable_pin, START); //values goe from 0 to 255, with 255 being a 100% duty cycle

    
     int down_count=0;
     while((down_count<200)  && (digitalRead(bottom_limit_pin)==HIGH))
     {
       //delay(100);
       down_count++;
       stepper_motor(STEP_DOWN);
     }
     
     //stop motor
    // analogWrite(motor_enable_pin,STOP);
     stepper_motor(STOP);
    
    height_down_previous_state=height_down; //update previous state
  }
  
  //####################################### end height control ###################################
  
  
  //add height return to this section#####################************************************************************
    //####################################### start home control ###################################
else if(home_state!=home_previous_state) //enter key hit, want to go to home
  {
    sendMountHome();
    home_previous_state=home_state; //update previous state
  }
  
  
  //####################################### end home control ###################################
  
  else //all states have not changed since last time function was called
  {
    //do nothing
  }
  
  
}


void stepper_motor(int STEP_UP_or_STEP_DOWN_or_STOP)  // start is high, stop is low
{
  // put your main code here, to run repeatedly: 
  switch (STEP_UP_or_STEP_DOWN_or_STOP) {
  case STEP_UP:
  digitalWrite(step_pin_1, LOW);
  digitalWrite(step_pin_2, LOW);
  digitalWrite(step_pin_3, LOW);
  digitalWrite(step_pin_4, HIGH);
  
  delay(2);
  
  digitalWrite(step_pin_1, LOW);
  digitalWrite(step_pin_2, LOW);
  digitalWrite(step_pin_3, HIGH);
  digitalWrite(step_pin_4, LOW);
  
  delay(2);
  
  digitalWrite(step_pin_1, LOW);
  digitalWrite(step_pin_2, HIGH);
  digitalWrite(step_pin_3, LOW);
  digitalWrite(step_pin_4, LOW);
  
  delay(2);
  
  digitalWrite(step_pin_1, HIGH);
  digitalWrite(step_pin_2, LOW);
  digitalWrite(step_pin_3, LOW);
  digitalWrite(step_pin_4, LOW);
  
  delay(2);
  
  break;
  
  case STEP_DOWN:
  digitalWrite(step_pin_1, HIGH);
  digitalWrite(step_pin_2, LOW);
  digitalWrite(step_pin_3, LOW);
  digitalWrite(step_pin_4, LOW);
  
  delay(2);
  
  digitalWrite(step_pin_1, LOW);
  digitalWrite(step_pin_2, HIGH);
  digitalWrite(step_pin_3, LOW);
  digitalWrite(step_pin_4, LOW);
  
  delay(2);
  
  digitalWrite(step_pin_1, LOW);
  digitalWrite(step_pin_2, LOW);
  digitalWrite(step_pin_3, HIGH);
  digitalWrite(step_pin_4, LOW);
  
  delay(2);
  
  digitalWrite(step_pin_1, LOW);
  digitalWrite(step_pin_2, LOW);
  digitalWrite(step_pin_3, LOW);
  digitalWrite(step_pin_4, HIGH);
  
  delay(2);
  
  break;
  
  case STOP:
  default:
  digitalWrite(step_pin_1, LOW);
  digitalWrite(step_pin_2, LOW);
  digitalWrite(step_pin_3, LOW);
  digitalWrite(step_pin_4, LOW);
  
  break;
  }}
